# Convert LaTeX formulas to MathJax: $...$ => \(...\) and $$...$$ => \[...\]. 
# Also convert \% to %
#
# INPUT:
#  text: a character string
#  lineNumbers: original line numbers of text (for error reporting)
#  undoBraceHide: If left/right braces have been converted to ctrl-a and ctrl-b, this
#                 can be undone here.
#
# VALUE: updated 'text', ready for mathJax
#
doMathJax = function(text, lineNumbers, undoBraceHide=FALSE) {
  text = gsub("\\$", "\f", text, fixed=TRUE) # hide plain dollar signs
  dollars = str_locate_all(text, "[$]{2}")
  # convert double dollars to \[...\]
  for (i in seq(along=dollars)) {
    mat = dollars[[i]]
    nrd = nrow(mat)
    if (nrd==0) next
    if (nrd %% 2 != 0 ) {
      stop("@", lineNumbers[i], " mismatched double dollar signs (or missing '\\' before '{' or '}')")
    }
    sel = rev(mat[seq(1, nrd, 2), "start"])
    for (j in sel) {
      tlen = nchar(text[i])
      if (j==1) {
        text[i] = paste0("\\[", substring(text[i], 3))
      } else {
        text[i] = paste0(substring(text[i], 1, j-1), 
                         "\\[", substring(text[i], j+2))
      }
    }
    text[i] = gsub("$$", "\\]", text[i], fixed=TRUE)
  }
  # convert single dollars to \(...\)
  dollars = str_locate_all(text, "[$]")
  for (i in seq(along=dollars)) {
    mat = dollars[[i]]
    nrd = nrow(mat)
    if (nrd==0) next
    if (nrd %% 2 != 0 ) {
      stop("@", lineNumbers[i], " mismatched dollar signs (or missing '\\' before '{' or '}')")
    }
    sel = rev(mat[seq(1, nrd, 2), "start"])
    for (j in sel) {
      tlen = nchar(text[i])
      if (j==1) {
        text[i] = paste0("\\(", substring(text[i], 2))
      } else {
       text[i] = paste0(substring(text[i], 1, j-1), 
                         "\\(", substring(text[i], j+1))
      }
    }
    text[i] = gsub("$", "\\)", text[i], fixed=TRUE)
  }
  text = gsub("\f", "$", text, fixed=TRUE) # restore plain dollar signs
  if (undoBraceHide) {
    text = gsub("\a", "{", text, fixed=TRUE)
    text = gsub("\b", "}", text, fixed=TRUE)
  }
  return(gsub("[\\][%]", "%", text))
}
