# feedbackCommon()
# common code called by rFeedback() and rHiddenFeedback()
# writes feedback function plus creates all but the beginning of the server code
# returns server code


feedbackCommon = function(inputId, text, isHidden, options, lineNumOffset) {
  
  temp = ifelse(isHidden, "RHidden", "rCodeFeedback")
  serverCode = c(paste0('output$', temp, 'Out', inputId, '=renderPrint({'),
                 paste0('  INPUT = readRCodeRtn', inputId, '()$input'),
                 paste0('  OUTPUT = readRCodeRtn', inputId, '()$output'),
                 paste0('  NEFARIOUS = readRCodeRtn', inputId, '()$nefarious'),
                 paste0('  EMPTY = length(INPUT)==0'),
                 paste0('  ERRORMESSAGE = readRCodeRtn', inputId, '()$errorMessage'),
                 paste0('  ERROR = length(ERRORMESSAGE)!=0'),
                 paste0('  ERRORCODE = readRCodeRtn', inputId, '()$errorCode'),
                 paste0('  VARS = readRCodeRtn', inputId, '()$VARS'),
                 paste0('  VARNAMES = names(VARS)'),
                 '',
                 paste0('  doit = try(', ifelse(isHidden, "hiddenF", 'f'),'eedbackFor', inputId,
                        '(INPUT, OUTPUT, NEFARIOUS, EMPTY, ERRORMESSAGE, ERROR, ERRORCODE, VARS, VARNAMES), silent=TRUE)'),
                 '  if (is(doit, "try-error")) {',
                 paste0('    write(paste(Sys.time(), "', inputId, '", ',
                        'attr(doit, "condition")$message), file="AUTHORERRORLOG.txt", append=TRUE)'),
                 '    cat("> Sorry, the feedback algorithm failed. Try entering something else. <")',
                 '  }',
                 '})')
  
  ###### Create feedback function in feedback function file for this input #####
  # function to create text for one section of a giant if () { foo } else if { bar } else if { spam } ...
  ifBlock = function(textCondition, text, type=c("elseif", "if", "else", "naked"), indent=1) {
    type = match.arg(type)
    indent = paste(rep("  ",indent), collapse="")
    indent2 = paste0("  ", indent)
    if (type == "if") {
      rtn = paste0(indent, "if (", textCondition, ") {")
    } else if (type=="elseif") {
      rtn = paste0(indent, "} else if (", textCondition, ") {")
    } else if (type=="else") {
      rtn = paste0(indent, "} else {")
    } else { # "naked"
      rtn = NULL
    }
    if (is.null(text)) return(rtn)
    return(c(rtn, paste0(indent2, text, sep="")))
  }
  
  
  #### Parse the "recipies" for feedback ####
  parseList = feedbackParser(text, lineNumOffset)
  groupInfo = parseList[["groupInfo"]]
  nGroups = nrow(groupInfo)
  groupDetail = parseList[["groupDetail"]]
  empty = parseList[["empty"]]
  nefarious = parseList[["nefarious"]]
  globalDefault = parseList[["globalDefault"]]
  rm(parseList)
  
  # Convenience function
  appendOut = function(x) write(x, file=outFile, append=TRUE)
  
  #### write the file with the feedback function ####
  temp = ifelse(isHidden, "hiddenF", "f")
  outFile = paste0(temp, 'eedbackFor', inputId, '.R')
  write(c(paste0('# ', temp, 'eedbackFor', inputId, '()'),
          '',
          paste0(temp, 'eedbackFor', inputId, ' = function(INPUT, OUTPUT, NEFARIOUS, EMPTY, ERRORMESSAGE, ERROR, ERRORCODE, VARS, VARNAMES) {'),
          '  groupHits = NULL',
          ''),
        file = outFile)
  
  appendOut(c('  if (EMPTY) {',
              ifBlock(NULL, empty, "naked"),
              '  } else if (NEFARIOUS) {',
              ifBlock(NULL, nefarious, "naked"),
              '  } else {',
              ''))
  
  
  # Make one big if() for each group
  for (group in 1:nGroups) {
    # Optional user error log
    if (group == 1 && options$monitorAll) {
      if (options$monitorErrors) {
        appendOut(c('    if (!ERROR && INPUT != "") {',
                    paste0('      write(paste(Sys.time(), "', inputId, '", ', 'paste(INPUT, collapse="; ")),'),
                    '        file="USERERRORLOG.txt", append=TRUE)',
                    '    }', ''))
      } else {
        appendOut(c('    if (INPUT != "") {',
                    paste0('      write(paste(Sys.time(), "', inputId, '", ', 'paste(INPUT, collapse="; ")),'),
                    '        file="USERERRORLOG.txt", append=TRUE)',
                    '    }', ''))
      }
    }
    
    # Optional user error monitoring
    if (group == 1 && options$monitorErrors) {
      appendOut(c('    if (ERROR) {',
                  paste0('      write(c(paste(Sys.time(), "', inputId, '", ',
                         'paste(ERRORCODE, collapse="; ")),'),
                  '              paste(ERRORMESSAGE, collapse="; ")),', 
                  '            file="USERERRORLOG.txt", append=TRUE)',
                  '    }', ''))
    }
    
    # Write code for one group
    appendOut(paste("    # Group", group))
    
    appendOut(paste0('    if (', 
                     ifelse(groupInfo[group, "isErrorGroup"], 'ERROR && ',
                            ifelse(groupInfo[group, "isNoErrorGroup"],  '!ERROR && ', '')),
                     '(', groupInfo[group, "condition"], ')) {'))
    appendOut('      groupHit = TRUE')
    gD = groupDetail[[group]]
    # 
    # Handle optional CALCulation section
    if (gD[[1]]$condition == 'CALC') {
      appendOut(ifBlock(NULL, gD[[1]]$feedback, "naked", indent=2))
      gD = gD[-1]
    }
    #
    # Write code for all remaining sections
    for (section in 1:length(gD)) {
      cond = gD[[section]]$condition
      text = gD[[section]]$feedback
      if (cond != "DEFAULT") {
        cond = try(makeCondition(cond))
        if (is(cond, "try-error")) {
          stop(attr(cond, "message"), " @ ", lineNumOffset + groupInfo[group, "groupStart"] - 1 + 
                 gD[[section]]$line)
        }
        if (cond == "CALC") {
          appendOut(ifBlock(cond, text, type=ifelse(section==1, "if", "elseif"), indent=2))
        }
        appendOut(ifBlock(cond, text, type=ifelse(section==1, "if", "elseif"), indent=3))
      } else { # this is a DEFAULT
        if (section == 1) appendOut(ifBlock("TRUE", NULL, "if", indent=3))
        appendOut(ifBlock(NULL, text, ifelse(section==1, "naked", "else"), indent=3))
      }
    } # end for each section
    if (gD[[length(gD)]]$condition != "DEFAULT") {
      appendOut(ifBlock(NULL, 'groupHit = FALSE', "else", indent=3))
    }
    appendOut(c("      }",
                "      groupHits = c(groupHits, groupHit)",
                "    }",
                ""))
  } # end for each group
  
  
  # Complete file with feedback function
  write(c('    if (!any(groupHits)) {',
          paste0('      ', globalDefault),
          '    }',
          '',
          '  }',
          '',
          '  return(NULL)',
          '}'), 
        file=outFile, append=TRUE)
  
  return(serverCode)
}
