# Function for finding right parenthesis using a silly, slow parse approach
# INPUT: 
#  txt: text like "INPUTHAS(abc) || OUTPUTHAS(def))"
#  noRE: FALSE prevents [(] from looking like a parenthesis
# OUTPUT: a list with 'rhLoc' as the starting location of the right hand side not used (or NA if at 
#         the end) and 'inside' as the inside of the parenthesized option of the first all upper
#         case text (must start with an upper case letter) or a "try-error" class object if failing.
#
# Used by makeCondition()
#
getParen = function(txt, noRE=FALSE) {
  tryError = list(); class(tryError)="try-error"
  start = regexpr("^[A-Z]+[*]?", txt)
  dropped = attr(start, "match.length")
  hasStar = (substring(txt, dropped, dropped) == "*")
  txt = substring(txt, dropped + 1)
  lp = regexpr("^[[:space:]]*[(]", txt)
  if (lp == -1) return(list(rhLoc = dropped + 1, inside = ""))
  dropped = dropped + attr(lp, "match.length")
  txt = substring(txt, attr(lp, "match.length") + 1)
  nct = nchar(txt)
  if (nct==0) {
    attr(tryError, "message") = "malformed parenthesized condition"
    return(tryError)
  }
  chars = strsplit(txt, split="")[[1]]
  i = 1
  OK = FALSE
  inDQ = FALSE # in double quotes
  inSQ = FALSE # in single quotes
  inSB = FALSE # in square brackets (for regular expressions)
  depth = 1
  while (i<=nct) {
    if (inDQ) {
      if (chars[i] == '"') {
        inDQ = FALSE
      }
    } else if (inSQ) {
      if (chars[i] == "'") {
        inSQ = FALSE
      }
    } else if (inSB) {
      if (chars[i] == ']') {
        inSB = FALSE
      }
    } else if (chars[i] == '"') {
      inDQ = TRUE
    } else if (chars[i] == "'") {
      inSQ = TRUE
    } else if (!noRE && chars[i] == "[") {
      inSB = TRUE
    } else if (chars[i] == "(") {
      depth = depth + 1
    } else if (chars[i] == ")") {
      depth = depth - 1
      if (depth==0) {
        OK = TRUE
        break
      }
    }
    i = i + 1
  } # end while looking for end ")"
  
  if (!OK) {
    attr(tryError, "message") = "malformed parenthesized condition"
    return(tryError)
  }
  return(list(rhLoc = dropped + i + 1, inside = substring(txt, 1, i-1), hasStar=hasStar))
} # end getParen()
